@@ -31,7 +31,7 @@ int launch_program (char* maskstr, ...);
 int get_lastrc();
 int get_timeforcmd();
 void set_timeforcmd(int timeout);
-int check_permissions (char *file, int uid, int mask);
+int check_permissions (char *file, int uid, int perms, int mask);
 int get_owner (char *file);
 int get_permissions (char *file);
 int keydelayms (struct timeval *start, int value, int delay);
