@@ -157,6 +157,8 @@
 #define TAG_HEARTBEATBEEP		MOD_POWER | 24
 #define TAG_POLICY				MOD_POWER | 26
 #define TAG_GOTOHIBERNATE		MOD_POWER | 27
+#define TAG_IBAMDATADIR			MOD_POWER | FLG_STRING | 28 
+#define TAG_CHARGETIME			MOD_POWER | 29
 
 #define TAG_ONAC_POLICY			MOD_POWER | 50
 #define TAG_ONAC_TIMERACTION	MOD_POWER | 51
