@@ -141,6 +141,10 @@ static struct tagtable tagtab[] = {
   { "TAG_NETLOADMIN",        TAG_NETLOADMIN },
   { "TAG_NETLOADPERIOD",     TAG_NETLOADPERIOD },
   { "TAG_NETLOADDEV",        TAG_NETLOADDEV },
+#ifdef WITH_IBAM
+  { "TAG_IBAMDATADIR",       TAG_IBAMDATADIR },
+  { "TAG_CHARGETIME",        TAG_CHARGETIME },
+#endif
   { "TAG_POLICY",            TAG_POLICY },
   
   { "TAG_ONAC_POLICY",       TAG_ONAC_POLICY },
