@@ -20,6 +20,7 @@
 #include <string.h>
 #include "pbbinput.h"
 
+#include <glib.h>
 #include <pbb.h>
 
 #include "gettext_macros.h"
@@ -32,6 +33,8 @@ extern volatile sig_atomic_t prgexit;
 /* --- private module data structure of the input manager --- */
 
 struct moddata_inputmanager {
+	GList		*inputsources;
+	
 	struct {
 		unsigned int shift:1;
 		unsigned int ctrl:1;
@@ -39,16 +42,6 @@ struct moddata_inputmanager {
 		unsigned int :0;
 	} flags;
 	unsigned short lastkey;
-	struct {
-		int fd;
-		int (*handler)(int fd);
-	} input[MAXINPUTS+1];
-	struct {
-		short timer;
-		short start;
-		void (*handler)();
-	} timer[TIMERCOUNT];
-	struct evdevice evdevs[EVDEVCOUNT];
 	inputqueue_t *iqueues[QUEUECOUNT];
 	inputqueue_t kbdqueue[MODULECOUNT];
 	inputqueue_t mousequeue[MODULECOUNT];
@@ -63,34 +56,16 @@ struct moddata_inputmanager {
 /* --- interface functions of the inputmanager --- */
 
 /* Init function called by the program init procedure. Function is known by
-   the prginitab. It does module data initialisation. */
-
+ * the prginitab. It does module data initialisation.
+ */
 int
 inputmanager_init ()
 {
 	struct moddata_inputmanager *base = &modbase_inputmanager;
-	int x, y, n;
-
-	for (n=0; n <= MAXINPUTS; n++) {
-    	base->input[n].fd = -1;
-    	base->input[n].handler = NULL;
-	}
-	for (n=0; n < EVDEVCOUNT; n++) {
-		base->evdevs[n].product = 0;
-		base->evdevs[n].vendor  = 0;
-		base->evdevs[n].inputid = -1;
-	}
+	int x, y;
 
 	scanEventDevices();
 	
-    /* each of TIMERCOUNT (=3) timers have to be initialised here.
-	   there is no mechanism to check if a timer slot is empty */
-	base->timer[0].handler = timer10_handler;  /* timer for 10ms queue */
-	base->timer[1].start = 10;
-	base->timer[1].handler = timer100_handler;  /* timer for 100ms queue */
-	base->timer[2].start = 100;
-	base->timer[2].handler = timer1000_handler;  /* timer for 1s queue */
-
 	base->flags.shift = 0;
 	base->flags.ctrl = 0;
 	base->flags.alt = 0;
@@ -112,15 +87,19 @@ inputmanager_init ()
 	return 0;
 }
 
+void
+cbFreeInputSource (gpointer data, gpointer user_data)
+{
+	InputSource *src = data;
+	g_source_remove (src->watch);
+}
+
 int
 inputmanager_exit ()
 {
 	struct moddata_inputmanager *base = &modbase_inputmanager;
-	int n;
 
-	for (n=0; n < MAXINPUTS; n++)      /* close all input files */
-		if (base->input[n].fd != -1)
-			close (base->input[n].fd);
+	g_list_foreach (base->inputsources, cbFreeInputSource, NULL);
 	return 0;
 }
 
@@ -157,168 +136,118 @@ get_modifier()
   return (base->flags.ctrl << 2) | (base->flags.alt << 1) | (base->flags.shift << 0);
 }
 
-/* --- timer function --- */
-
-void
-call_timertick ()
-{
-	struct moddata_inputmanager *base = &modbase_inputmanager;
-	int n;
-
-	for (n=0; n < TIMERCOUNT; n++) {
-		if ((--(base->timer[n].timer)) <= 0) {
-			base->timer[n].timer = base->timer[n].start;
-			base->timer[n].handler();
-		}
-	}
-}
-
 /* --- event handler management --- */
 
-struct evdevice*
-findEventDevice (int fd)
+gint
+cbEventDevices (gconstpointer a, gconstpointer venprod)
 {
-	struct moddata_inputmanager *base = &modbase_inputmanager;
-	int n;
-
-	for (n = 0; n < EVDEVCOUNT; n++)
-		if (fd == base->evdevs[n].inputid)
-			return &base->evdevs[n];
-		
-	return NULL;
-}
-
-void
-clearEventDevice (struct evdevice *evdev)
-{
-	if (evdev->inputid >= 0) {
-		unregister_inputhandler(evdev->inputid);
-		close(evdev->inputid);
-#ifdef DEBUG
-		printf("Inputhandler unregistered:\t%04x %04x         \n",
-		    evdev->product, evdev->vendor);
-#endif
-	}
-	evdev->product = 0;
-	evdev->vendor  = 0;
-	evdev->inputid = -1;
+	const InputSource *src = a;
+	
+	if (src->user_data == venprod)
+		return 0;
+	return 1;
 }
 
 void
 scanEventDevices ()
 {
 	struct moddata_inputmanager *base = &modbase_inputmanager;
-
-	int n, m, fd, fcnt = 0;
-	char filename[20];
 	unsigned short id[4] = { 0,0,0,0 };
+	int n, fd, fcnt = 0;
+	char filename[20];
+	gpointer evdevid;
 
-	for (n = 0, m = 0; n < EVDEVCOUNT; n++) {
+	for (n = 0; n < EVDEVCOUNT; n++) {
 		sprintf(filename, "/dev/input/event%d", n);
 		if ((fd = open(filename, O_RDONLY)) >= 0) {
-			ioctl(fd, EVIOCGID, id);
-			if (id[ID_PRODUCT] != base->evdevs[m].product ||
-				id[ID_VENDOR]  != base->evdevs[m].vendor) {
-				clearEventDevice (&base->evdevs[m]);
-				base->evdevs[m].inputid = fd;
-				base->evdevs[m].product = id[ID_PRODUCT];
-				base->evdevs[m].vendor = id[ID_VENDOR];
-				register_inputhandler(fd, event_handler);
-#ifdef DEBUG
-				printf("Inputhandler registered:\t%04x %04x         \n",id[ID_PRODUCT],id[ID_VENDOR]);
-#endif
-			} else
+			evdevid = (gpointer) (0x45460000 | n);
+			if ((g_list_find_custom (base->inputsources, evdevid, cbEventDevices)) == NULL)
+				addInputSource (fd, event_handler, evdevid, TRUE);
+			else
 				close(fd);
-			m++;
 		} else
 			fcnt++;  /* count number of event devices that could not be opened */
 	}
 
-	for (; m < EVDEVCOUNT; m++)    /* cleanup rest of list */
-		clearEventDevice (&base->evdevs[m]);
-
 	if (fcnt == EVDEVCOUNT)  /* none of the 32 devices can be opened */
 		print_msg (PBB_WARN, _("No event devices available. Please check your configuration.\n"));
 }
 
-
-/* --- input handler --- */
-
-int
-register_inputhandler (int fd, void *func)
-{
-	struct moddata_inputmanager *base = &modbase_inputmanager;
-	int n;
-
-	for (n=0; n < MAXINPUTS; n++)
-		if (base->input[n].fd == -1) {
-			base->input[n].fd = fd;
-      		base->input[n].handler = func;
-      		return 0;
-    	}
-	return -1;
-}
+/* --- input source --- */
 
 void
-unregister_inputhandler (int fd)
+destroyInputSource (gpointer data)
 {
 	struct moddata_inputmanager *base = &modbase_inputmanager;
-	int n, found = 0;
-
-	for (n=0; n <= MAXINPUTS; n++)
-		if (found) {
-			base->input[n-1].fd = base->input[n].fd;
-			base->input[n-1].handler = base->input[n].handler;
-		} else if (base->input[n].fd == fd)
-			found = 1;
-	if (found)
-		base->input[n].fd = -1;
-}	
-
-int
-create_fdset (fd_set *watchset)
-{
-	struct moddata_inputmanager *base = &modbase_inputmanager;
-	int n, maxfd;
-
-	FD_ZERO(watchset);
-	for (maxfd=n=0; n < MAXINPUTS; n++) {
-		if (base->input[n].fd == -1) break;
-		FD_SET(base->input[n].fd, watchset);
-		if (base->input[n].fd > maxfd)
-			maxfd = base->input[n].fd;
-	}
-	return maxfd;
+	InputSource *src = data;
+#ifdef DEBUG
+	print_msg (PBB_INFO, "DBG: InputSource removed: %08x\n", src->user_data);
+#endif
+	base->inputsources = g_list_remove (base->inputsources, src);
+	g_free (src);
 }
 
-void
-call_inputhandler(fd_set *inset)
+gboolean
+handleInputSource (GIOChannel *io, GIOCondition condition, gpointer data)
 {
-	struct moddata_inputmanager *base = &modbase_inputmanager;
-	struct evdevice *evdev;
-	int n = 0, rescan = 0;
-
-	while (n < MAXINPUTS && base->input[n].fd != -1) {
-		if (FD_ISSET(base->input[n].fd, inset)) {
-			if ((base->input[n].handler (base->input[n].fd)) == -1) {
-		    	if ((evdev = findEventDevice (base->input[n].fd))) {
-					clearEventDevice (evdev);
-					rescan = 1;
-				} else {
-					print_msg (PBB_ERR,
-							_("Can't read filehandle %d anymore ... exiting!\n"),
-							base->input[n].fd);
-					prgexit = 1;
-					return;
-				}
-			} else
-				n++;
-		} else
-			n++;
-	}
+	InputSource *src = data;
+	gboolean rc = TRUE;
 	
-	if (rescan)     /* one or more filehandles have failed and must be closed */
-		scanEventDevices ();  /* so rescan the event devices to fill the gap */
+	if (condition & G_IO_IN) {
+		/* call the appropriate handler. We assume that io = src->io.
+		 * In case of an error the InputSource will be removed to
+		 * prevent further problems reading invalid devices. */
+		rc = src->handler (g_io_channel_unix_get_fd (io), src->user_data);
+	} else
+		/* GIOChannel reported an error */
+		rc = FALSE;
+
+#ifdef DEBUG
+	if (rc == FALSE) {
+		print_msg (PBB_INFO, "DBG: InputSource received %s %s %s\n",
+				(condition & G_IO_IN)  ? "G_IO_IN"  : "",
+		        (condition & G_IO_ERR) ? "G_IO_ERR" : "",
+		        (condition & G_IO_HUP) ? "G_IO_HUP" : "");
+	}
+#endif
+	return rc;
+}
+
+InputSource*
+addInputSource (int fd, void *handler, gpointer user_data, gboolean close_on_exit)
+{
+	struct moddata_inputmanager *base = &modbase_inputmanager;
+	InputSource *src;
+
+	src = g_new0 (InputSource, 1);
+	src->io = g_io_channel_unix_new (fd);
+	
+	/* set channel encoding to binary */
+	g_io_channel_set_encoding (src->io, NULL, NULL);
+
+	/* Bound the file handle to the IOChannel. The file handle will be
+	 * closed as soon as this IOChannel got obsolete. If this shouldn't
+	 * happen, give 'close_on_exit' = FALSE. */
+	if (close_on_exit == TRUE)
+		g_io_channel_set_close_on_unref (src->io, TRUE);
+	
+	src->handler   = handler;
+	src->user_data = user_data;
+	src->watch   = g_io_add_watch_full (src->io, G_PRIORITY_DEFAULT,
+			G_IO_IN | G_IO_ERR | G_IO_HUP, handleInputSource, src,
+			destroyInputSource);
+
+	/* g_io_channel_unix_new() and g_io_add_watch both increment the
+	 * ref counter but we want to get rid of the IOchannel too, when
+	 * the source will be removed. */
+	g_io_channel_unref (src->io);
+
+	/* Attach the newly created InputSource to the global list */
+	base->inputsources = g_list_append (base->inputsources, src);
+#ifdef DEBUG
+	print_msg (PBB_INFO, "DBG: InputSource added:   %08x\n", src->user_data);
+#endif
+	return src;
 }
 
 /* --- queue managers --- */
@@ -402,17 +331,18 @@ ipc_handler ()
 	}
 }
 
-int
-event_handler (int fd)
+gboolean
+event_handler (int fd, gpointer user_data)
 {
 	struct input_event inp;
 	struct tagitem taglist[10];
-
+	gboolean rc = FALSE;
+	
 	taglist_init(taglist);
 	
 	if (read(fd, &inp, sizeof(inp)) == sizeof(inp)) {
 		switch (inp.type) {
-		case EV_KEY:		
+		case EV_KEY:
 			taglist_add (taglist, TAG_KEYCODE,  (long) inp.code);
 			taglist_add (taglist, TAG_KEYREPEAT,(long) inp.value);
 			taglist_add (taglist, TAG_MODIFIER, (long) track_modifier (inp.code, inp.value));
@@ -428,20 +358,20 @@ event_handler (int fd)
     		process_queue (MOUSEQUEUE, taglist);
 			break;
 		case EV_ABS:
-			/* Mouse events are not used. Only the occourence of this events
+			/* Mouse events are not used. Only the occurence of this events
 			 * will currently be used */
     		process_queue (MOUSEQUEUE, taglist);
 			break;
 		default:
 			break; /* catch unknown event types */
 		}
-		return 0;
-	} else
-		return -1; /* read has failed */
+		rc = TRUE;
+	}
+	return rc;
 }
 
-void
-timer10_handler ()
+gboolean
+cb_timer10 (gpointer data)
 {
   struct moddata_inputmanager *base = &modbase_inputmanager;
   int mod;
@@ -452,18 +382,23 @@ timer10_handler ()
   mod = get_modifier();
   if ((mod == 0) && (base->lastkey == 0))
     process_queue (SECUREQUEUE, taglist);
+  
+  return TRUE;
 }
 
-void
-timer100_handler ()
+gboolean
+cb_timer100 (gpointer data)
 {
   struct tagitem taglist[] = { { TAG_END, 0 } };
   process_queue (T100QUEUE, taglist);
+  return TRUE;
 }
 
-void
-timer1000_handler ()
+gboolean
+cb_timer1000 (gpointer data)
 {
   struct tagitem taglist[] = { { TAG_END, 0 } };
   process_queue (T1000QUEUE, taglist);
+  return TRUE;
 }
+
