@@ -12,6 +12,7 @@
  * 2 of the License, or (at your option) any later version.
  * ----------------------------------------------------------------------------*/
 
+#include <glib.h>
 #include <pbbtaglist.h>
 
 typedef void (*inputqueue_t)();
@@ -47,32 +48,32 @@ typedef void (*inputqueue_t)();
 #define LONG(x) ((x)/BITS_PER_LONG)
 #define test_bit(bit, array) ((array[LONG(bit)] >> OFF(bit)) & 1)
 
-struct evdevice {
-	int inputid; /* in fact the filehandle to identify the input handler */
-	unsigned short vendor;
-	unsigned short product;
-};
+typedef struct inputsource {
+	GIOChannel *io;
+	guint watch;
+	int (*handler)(int fd, gpointer user_data);
+	gpointer user_data;
+} InputSource;
 
 /* public prototypes */
 int inputmanager_init ();
+void cbFreeInputSource (gpointer data, gpointer user_data);
 int inputmanager_exit ();
-int register_inputhandler (int fd, void *func);
-void unregister_inputhandler (int fd);
-int register_function (int queueid, void *func );
-long process_queue_single (int queueid, long tag, long data);
-void process_queue (int queueid, struct tagitem *taglist);
-int create_fdset (fd_set *watchset);
-void call_inputhandler(fd_set *inset);
-void call_timertick ();
-
- /* private prototypes */
-void scanEventDevices ();
 int track_modifier(unsigned short code, unsigned int value);
 int get_modifier();
+
+gint cbEventDevices (gconstpointer a, gconstpointer b);
+void scanEventDevices ();
+void destroyInputSource (gpointer data);
+gboolean handleInputSource (GIOChannel *io, GIOCondition condition, gpointer data);
+InputSource *addInputSource (int fd, void *handler, gpointer user_data, gboolean close_on_exit);
+int register_function (int queueid, void *func);
+long process_queue_single (int queueid, long tag, long data);
+void process_queue (int queueid, struct tagitem *taglist);
 void ipc_handler ();
-int event_handler (int fd);
-void timer10_handler ();
-void timer100_handler ();
-void timer1000_handler ();
+gboolean event_handler (int fd, gpointer user_data);
+gboolean cb_timer10 (gpointer data);
+gboolean cb_timer100 (gpointer data);
+gboolean cb_timer1000 (gpointer data);
 
 #endif    /* INCLUDE_INPUTMANAGER_H */
