@@ -313,7 +313,8 @@ display_keyboard (struct tagitem *taglist)
 				if (tmp + base->lcdlight.offset > base->lcdlight.max)
 					base->lcdlight.offset = base->lcdlight.max - tmp;
 				if (tmp + base->lcdlight.offset < 1)
-					base->lcdlight.offset = -tmp + 1;
+					base->lcdlight.offset = -tmp;
+/*	CHECK:			base->lcdlight.offset = -tmp + 1; */
 				level = tmp + base->lcdlight.offset;
 			}
 			level = display_set_lcdbrightness(level);
@@ -421,7 +422,10 @@ display_ambienttimer (struct tagitem *taglist)
 				display_set_kbdbrightness(level);
 			}
 			
-			if (base->flags.lcd_autoadjust) {
+			/* CHECK: */
+			/* skip the autoadjustment if the current brightness level is zero.
+			 * It's users will and we don't want to interfere with it */
+			if (base->flags.lcd_autoadjust && base->lcdlight.current != 0) {
 				level = (base->lcdlight.max * ambient/base->lcdlight.threshold) + 1;
 				if (level > lcdmax) level = lcdmax;
 				if (level < lcdmin) level = lcdmin;
@@ -625,6 +629,7 @@ display_handle_tags (int cfgure, struct tagitem *taglist)
 			}
 			break;
 		case TAG_WAKEUPFROMSLEEP:  /* private tag */
+			display_sync_lcdbrightness ();
 			if ((cfgure) && base->lcdlight.current) {
 				display_framebuffer (1);
 			}
