@@ -151,6 +151,8 @@ static struct tagtable tagtab[] = {
   { "TAG_NETLOADMIN",        TAG_NETLOADMIN },
   { "TAG_NETLOADPERIOD",     TAG_NETLOADPERIOD },
   { "TAG_NETLOADDEV",        TAG_NETLOADDEV },
+  { "TAG_IBAMDATADIR",       TAG_IBAMDATADIR },
+  { "TAG_CHARGETIME",        TAG_CHARGETIME },
   { "TAG_POLICY",            TAG_POLICY },
   { "TAG_ONAC_POLICY",       TAG_ONAC_POLICY },
   { "TAG_ONAC_TIMERACTION",  TAG_ONAC_TIMERACTION },
