@@ -30,6 +30,7 @@
 #include <linux/pmu.h>
 #include <linux/adb.h>
 
+#include <glib.h>
 #include <pbb.h>
 
 #include "gettext_macros.h"
@@ -49,6 +50,7 @@ struct moddata_pmac {
 	int fd_pmu;      /* filehandle of the PMU device */
 	char *adbdev;    /* pathname of the ADB device */
 	int fd_adb;      /* filehandle of ADB device */
+	char *i2cdev;    /* pathname of the I2C device with ambient sensor and KBD illu */
 
 	int version;     /* PMU version */
 	char *identity;  /* Identity string of this laptop */
@@ -75,7 +77,6 @@ struct moddata_pmac {
 	int keyboard_mode;             /* keyboard mode for recent keyboards (0xC[0..4]) */
 	int batlog_mode;
 	int batlog_cycle;
-	int i2c_bus_lmu;
 	int lcdfeedback;               /* correction factor for ambient light sensor */
 #if defined(DEBUG) && SIMUAMBIENT
 	int ambient;
@@ -88,15 +89,18 @@ pmac_init ()
 	struct moddata_pmac *base = &modbase_pmac;
 	static char devbuffer_pmu[STDBUFFERLEN];
 	static char devbuffer_adb[STDBUFFERLEN];
+	static char devbuffer_i2c[STDBUFFERLEN];
 	static char identitystring[STDBUFFERLEN];
 	int sid;
 	
 	bzero (base, sizeof (struct moddata_pmac));
 	bzero (devbuffer_pmu, STDBUFFERLEN);
 	bzero (devbuffer_adb, STDBUFFERLEN);
+	bzero (devbuffer_i2c, STDBUFFERLEN);
 	bzero (identitystring, STDBUFFERLEN);
 	base->pmudev   = devbuffer_pmu;
 	base->adbdev   = devbuffer_adb;
+	base->i2cdev   = devbuffer_i2c;
 	base->identity = identitystring;
 
 	if ((sid = registerCfgSection ("MODULE PMAC")) == 0) {
@@ -115,8 +119,8 @@ pmac_init ()
 				N_("'"TRACKPAD_NOTAP_NAME"', '"TRACKPAD_TAP_NAME"', '"TRACKPAD_DRAG_NAME"' or '"TRACKPAD_LOCK_NAME"'"),
 				TRACKPAD_NOTAP_NAME, TRACKPAD_TAP_NAME, TRACKPAD_DRAG_NAME, TRACKPAD_LOCK_NAME, NULL);
 		registerCfgOptionList (sid, "KBDMode", TAG_KBDMODE, 0,
-				N_("'"KEYBOARD_FNBACK_NAME"' or '"KEYBOARD_FNTOP_NAME"'"),
-				KEYBOARD_FNBACK_NAME, KEYBOARD_FNTOP_NAME, NULL);
+				N_("'"KEYBOARD_FNDISABLED_NAME"', '"KEYBOARD_FNBACK_NAME"' or '"KEYBOARD_FNTOP_NAME"'"),
+				KEYBOARD_FNDISABLED_NAME, KEYBOARD_FNBACK_NAME, KEYBOARD_FNTOP_NAME, NULL);
 		registerCfgOptionList (sid, "Batlog", TAG_BATLOG, 0,
 				N_("'"BATLOG_NONE_NAME"', '"BATLOG_CYCLE_NAME"' or '"BATLOG_LOG_NAME"'"),
 				BATLOG_NONE_NAME, BATLOG_CYCLE_NAME, BATLOG_LOG_NAME, NULL);
@@ -155,8 +159,8 @@ pmac_open (struct tagitem *taglist)
 	base->keyboard_mode	= -1;
 	base->batlog_mode	= BATLOG_NONE;  /* no batlog */
 	base->batlog_cycle	= -1;         /* cycle invalid */
-	base->i2c_bus_lmu   = -1;       /* not available by default */
 	base->timeleft      = -1;
+	base->machine       = pmac_getmachine ();
 #if defined(DEBUG) && SIMUAMBIENT
 	base->ambient       = 0;
 #endif
@@ -179,16 +183,29 @@ pmac_open (struct tagitem *taglist)
 			return E_OPEN;
 		}
 	} else return rc;
-	
-	if ((val = lmu_find_bus()) >= 0) {
-		base->i2c_bus_lmu = val;
+
+	/* If this PowerBook has an ambient light sensor connected
+	 * to the I2C bus, search for the device and save the name
+	 */
+	val = -1;
+	if (haveI2CAmbient()) {
+		if ((val = findI2CBus()) >= 0)
+			snprintf(base->i2cdev, STDBUFFERLEN, "/dev/i2c-%d", val);
+		else if (val == (-E_PERM))
+			print_msg (PBB_WARN, _("Can't access i2c devices. Permission denied.\n"));
+	}
+
+	/* If we have an Ambient light sensor in this PowerBook, we
+	 * must measure the LCD feedback to this sensor in current
+	 * state. To do this we switch the backlight off for a moment
+	 * to measure the real ambient light level.
+	 */
+	if ((val >= 0) || havePMUAmbient()) {  
 		print_msg (PBB_INFO, _("LMU found, ambient light sensor and keyboard illumination active.\n"));
-	} else if (val == (-E_PERM))
-		print_msg (PBB_WARN, _("Can't access i2c devices. Permission denied.\n"));
-#ifdef DEBUG
-	else
-		print_msg (PBB_INFO, "DBG: No LMU present in this ancient PowerBook.\n");
-#endif
+		val = pmac_get_lcdbacklight();
+		pmac_set_lcdbacklight (0);    /* measure LCD feedback */
+		pmac_set_lcdbacklight (val);  /* restore LCD backlight level */
+	}
 
 	base->version  = get_pmu_version(base->fd_adb);
 	pmac_identify(base->identity, base->version);
@@ -198,12 +215,15 @@ pmac_open (struct tagitem *taglist)
 	pmac_update_flags ();
 	pmac_update_batteryinfo ();
 	base->flagschanged = 0;
-	base->machine = pmac_getmachine ();
 
 #ifdef DEBUG
 	print_msg (PBB_INFO, "DBG: Machine:  %0x\n", base->machine);	
-    print_msg (PBB_INFO, "DBG: Keyboard: %s\n",  haveADBKeyboard(base->machine) ? "ADB" : "USB");
-    print_msg (PBB_INFO, "DBG: Trackpad: %s\n",  haveADBTrackpad(base->machine) ? "ADB" : "USB");
+    print_msg (PBB_INFO, "DBG: Keyboard: %s\n",
+			haveADBKeyboard () ? "ADB" : "USB");
+    print_msg (PBB_INFO, "DBG: Trackpad: %s\n",
+			haveADBTrackpad () ? "ADB" : "USB");
+    print_msg (PBB_INFO, "DBG: Ambient:  %s\n",
+			haveI2CAmbient () ? "I2C" : havePMUAmbient () ? "PMU" : "None");
 #endif
 	
 	/* Every frame buffer driver could register a backlight controller,
@@ -214,23 +234,17 @@ pmac_open (struct tagitem *taglist)
 	 * controller is installed or if we have to call our fallback functions.
 	 */
 	
-	if (base->i2c_bus_lmu != -1) {  
-		val = pmac_get_lcdbacklight();
-		pmac_set_lcdbacklight (0);    /* measure LCD feedback */
-		pmac_set_lcdbacklight (val);  /* restore LCD backlight level */
-	}
-		
-	base->trackpad_mode = trackpad_get_config(base->fd_adb); /* get trackpad mode */
+	base->trackpad_mode = trackpad_get_config(); /* get trackpad mode */
 
 	if (base->version != OHARE_PMU) {
 		ioctl(base->fd_pmu, PMU_IOC_GRAB_BACKLIGHT, &DUMMY);
-		base->keyboard_mode = keyboard_get_config(base->fd_adb);	/* get keyboard config*/
+		base->keyboard_mode = keyboard_get_config();	/* get keyboard config*/
 	}
 
-	if ((register_inputhandler (dup(base->fd_pmu), pmac_pmu_handler)) < 0)
+	if ((addInputSource (base->fd_pmu, pmac_pmu_handler, NULL, FALSE)) == NULL)
 		print_msg (PBB_WARN, _("Can't install PMU input handler. Some functionality may be missing.\n"));
 
-	/* register keyboard handler for trackpad control only if we have an ADB keyboard */
+	/* register keyboard handler for trackpad control only if we have an ADB trackpad */
 	if (haveADBTrackpad ())
 		register_function (KBDQUEUE, pmac_keyboard);
 	
@@ -281,7 +295,7 @@ pmac_keyboard (struct tagitem *taglist)
 		 */
 		if (code != BTN_LEFT && !isModifier (code, mod) && base->flags.notap_typing) {
 			if (base->trackpad_pause == 0) {
-				trackpad_set_config(base->fd_adb, TRACKPAD_NOTAP);
+				trackpad_set_config(TRACKPAD_NOTAP);
 #if defined(DEBUG) && TRACKPAD
 				printf ("Trackpad tapping disabled\r");
 				fflush (stdout);
@@ -301,7 +315,7 @@ pmac_keyboard (struct tagitem *taglist)
 				base->trackpad_mode = TRACKPAD_LAST;
 		} else return;
 
-		trackpad_set_config(base->fd_adb, base->trackpad_mode);
+		trackpad_set_config(base->trackpad_mode);
 		singletag_to_clients (CHANGEVALUE, TAG_TPMODE, base->trackpad_mode);
 	}
 }
@@ -314,7 +328,7 @@ pmac_mouse (struct tagitem *taglist)
 	/* trackpad tapping control */
 	if (base->trackpad_pause) {
 		base->trackpad_pause = 0;
-		trackpad_set_config(base->fd_adb, base->trackpad_mode);
+		trackpad_set_config(base->trackpad_mode);
 #if defined(DEBUG) && TRACKPAD
 		printf ("Trackpad tapping enabled \r");
 		fflush (stdout);
@@ -331,7 +345,7 @@ pmac_secure (struct tagitem *taglist)
 	
 	if (base->flags.set_kbdmode == 1) {
 		base->flags.set_kbdmode = 0;
-		keyboard_set_config(base->fd_adb, base->keyboard_mode);
+		keyboard_set_config(base->keyboard_mode);
 	} else if (base->flags.goto_sleep == 1) {
 		base->flags.goto_sleep = 0;
 		if (base->flags.sleepsupported) {
@@ -381,7 +395,7 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 	while (taglist->tag != TAG_END) {
 		switch (taglist->tag) {
 		case TAG_REINIT:      /* system tag */
-			trackpad_set_config(base->fd_adb, base->trackpad_mode);
+			trackpad_set_config(base->trackpad_mode);
 			base->flags.set_kbdmode = 1;     /* reset keyboard mode */
 			break;
 		case TAG_PMUDEVICE:
@@ -425,7 +439,7 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 		case TAG_TPMODE:
 			if (cfgure)	{
 				base->trackpad_mode = taglist->data > TRACKPAD_LAST ? 0 : taglist->data;
-				trackpad_set_config(base->fd_adb, base->trackpad_mode);
+				trackpad_set_config(base->trackpad_mode);
 				singletag_to_clients (CHANGEVALUE, TAG_TPMODE, base->trackpad_mode);
 			} else
 				taglist->data = base->trackpad_mode;
@@ -438,6 +452,10 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure) {
 				if (base->version != OHARE_PMU) {
 					base->keyboard_mode = taglist->data > KEYBOARD_LAST ? 0 : taglist->data;
+					if (haveADBKeyboard() && !taglist->data) {
+						print_msg (PBB_WARN, _("ADB keyboard can't disable fnmode, force mode to 'fkeyslast'.\n"));
+						base->keyboard_mode = KEYBOARD_FNBACK;
+					}
 					base->flags.set_kbdmode = 1;     /* request a secure function call */
 				}
 			} else
@@ -466,10 +484,12 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure)	tagerror (taglist, E_NOWRITE);
 			else		taglist->data = base->flags.sleepsupported;
 			break;
+#ifndef WITH_IBAM
 		case TAG_TIMEREMAINING:
 			if (cfgure)	tagerror (taglist, E_NOWRITE);
 			else		taglist->data = base->timeleft;
 			break;
+#endif
 		case TAG_POWERSOURCE:
 			if (cfgure)	tagerror (taglist, E_NOWRITE);
 			else		taglist->data = base->flags.ac_power;
@@ -487,7 +507,7 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			break;
 
 		case TAG_AMBIENTLIGHT:
-			if (base->i2c_bus_lmu < 0)
+			if (!base->i2cdev[0] && !havePMUAmbient())
 				tagerror (taglist, E_NOSUPPORT);
 			else if (cfgure)
 #if defined(DEBUG) && SIMUAMBIENT
@@ -496,11 +516,11 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 				tagerror (taglist, E_NOWRITE);
 #endif
 			else
-				taglist->data = lmu_get_ambient();
+				taglist->data = getAmbient();
 			break;
 		case TAG_KEYBLIGHTLEVEL:   /* private tag */
 			if (cfgure)
-				lmu_set_illumination (base->i2c_bus_lmu, taglist->data);
+				setKBDIllumination (taglist->data);
 			break;
 		case TAG_KEYBLIGHTMAX:     /* private tag */
 			if (cfgure)	;
@@ -511,8 +531,8 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 	}
 }
 
-int
-pmac_pmu_handler (int fd)
+gboolean
+pmac_pmu_handler (int fd, gpointer user_data)
 {
 	unsigned char intr[16];
 	struct tagitem taglist[4];
@@ -552,8 +572,8 @@ pmac_pmu_handler (int fd)
 		printf ("\n");
 #endif
 	}
-	return 0; /* we assume that the filehandle for /dev/pmu can't
-				 be closed from outside pbbuttonsd */
+	return TRUE; /* we assume that the filehandle for /dev/pmu can't
+                    be closed from outside pbbuttonsd */
 }
 
 void
@@ -570,20 +590,27 @@ pmac_timer (struct tagitem *taglist)
 		base->flagschanged = pmac_update_flags (); /* if not, get ac power and cover status */
 	pmac_update_batteryinfo ();
 
+#ifndef WITH_PMUD
+	/* The COVERSTATUS must be sent before POWERCHANGED because the
+	 * POWERCHANGED code in module_powersave.c rely on the correct
+	 * cover status.
+	 */
+	if (base->flagschanged & PMU_ENV_LID_CLOSED)
+		taglist_add (args, TAG_COVERSTATUS, base->flags.coveropen);
+#endif
+	
 	if (base->flagschanged & PMU_ENV_AC_POWER) {
 		taglist_add (args, TAG_POWERCHANGED, base->flags.ac_power);
 		if (!base->flags.ac_power)      /* running on Battery */
 			base->batlog_cycle++;   /* then increase cycle */
 	}
 	if (val != base->timeleft && !base->flags.ac_power) {
+#ifndef WITH_IBAM
 		taglist_add (args, TAG_TIMECHANGED, base->timeleft);
+#endif
 		batlog_write(base->timeleft);
 	}
 
-#ifndef WITH_PMUD
-	if (base->flagschanged & PMU_ENV_LID_CLOSED)
-		taglist_add (args, TAG_COVERSTATUS, base->flags.coveropen);
-#endif
 	if (base->version == OHARE_PMU) {         /* PowerBook 3400, etc */
 		if ((val = get_button(base->fd_adb, PMU_GET_VOLBUTTON)) >= 0) {
 			val = val > 64 ? 100 : val * 100 / 64;
@@ -607,7 +634,7 @@ pmac_timer (struct tagitem *taglist)
 	/* trackpad tapping control */
 	if (base->trackpad_pause) {
 		if ((--base->trackpad_pause) == 0) {
-			trackpad_set_config(base->fd_adb, base->trackpad_mode);
+			trackpad_set_config(base->trackpad_mode);
 #if defined(DEBUG) && TRACKPAD
 			printf ("Trackpad tapping enabled \r");
 			fflush (stdout);
@@ -649,9 +676,9 @@ activate_sleepmode ()
 	int fd;
 #endif
 
-	tpmode = trackpad_get_config (base->fd_adb);
+	tpmode = trackpad_get_config ();
 	if (base->version != OHARE_PMU)
-		kbdmode = keyboard_get_config (base->fd_adb);
+		kbdmode = keyboard_get_config ();
 	sync();
 #ifndef WITH_PMUD
 	/* PMU_IOC_SLEEP only succeeds if called as root.
@@ -659,12 +686,12 @@ activate_sleepmode ()
 	 * the mashine to sleep
 	 */
 	ioctl (base->fd_pmu, PMU_IOC_SLEEP, 0);    /* returns after next awake */
-	trackpad_set_config(base->fd_adb, tpmode);
+	trackpad_set_config(tpmode);
 #else
 	if (((fd = tcp_open("localhost", "pmud", 879)) < 0)) {
 		print_msg (PBB_WARN, _("Can't find pmud on port 879. Trigger PMU directly for sleep.\n"));
 		ioctl (base->fd_pmu, PMU_IOC_SLEEP, 0);     /* returns after next awake */
-		trackpad_set_config(base->fd_adb, tpmode);
+		trackpad_set_config(tpmode);
 	} else {
 		tcp_printf(fd, "sleep\n");
 
@@ -678,7 +705,7 @@ activate_sleepmode ()
 	}
 #endif
 	if (base->version != OHARE_PMU)
-		keyboard_set_config(base->fd_adb, kbdmode);
+		keyboard_set_config(kbdmode);
 }
 
 /* This function gets the battery status from the PMU kernel driver via
@@ -868,15 +895,15 @@ pmac_set_lcdbacklight (int val)
 	if (val == -1)   /* no backlight controller available */
 		return;
 	
-	ambient = lmu_get_raw_ambient(base->i2c_bus_lmu) - base->lcdfeedback;
+	ambient = getRawAmbient () - base->lcdfeedback;
 	ioctl(base->fd_pmu, PMU_IOC_SET_BACKLIGHT, &val);
-	base->lcdfeedback = val ? lmu_get_raw_ambient(base->i2c_bus_lmu) - ambient : 0;
+	base->lcdfeedback = val ? getRawAmbient () - ambient : 0;
 
 #if defined(DEBUG) && (TESTAMBIENT || SIMUAMBIENT)
-	ambient = lmu_get_raw_ambient(base->i2c_bus_lmu);
+	ambient = getRawAmbient ();
 	printf("LCD Level: %2d, Raw (%4d) - LCD Feedback (%4d) = Ambient (%4d or %2d%%)\n",
 		val, ambient, base->lcdfeedback, ambient - base->lcdfeedback,
-		lmu_get_ambient());
+		getAmbient());
 	fflush(stdout);
 #endif
 	return;
@@ -887,18 +914,24 @@ pmac_set_lcdbacklight (int val)
 void
 pmac_identify (char* buffer, int pmu)
 {
-  int idx;
-  char *version[] = {
-    N_("Unknown PowerBook"),
-    "PowerBook 2400/3400/3500",        /*   9  */
-    "G3 PowerBook Wallstreet",          /*  10  */
-    "1999 G3 PowerBook Lombard",         /*  11  */
-    "iBook/G3 PB Pismo/G4 PB Titanium",   /*  12  */
-    N_("Wow, later than an Titanium! ;-)")
-  };
-  idx = pmu < 8 ? 8 : pmu - 8;
-  if (idx > 5) idx = 5;
-  snprintf(buffer, STDBUFFERLEN, "%s (PMU version: %d)", version[idx], pmu);
+	int idx;
+	char *version[] = {
+		N_("Unknown PowerBook"),
+		"PowerBook 2400/3400/3500",        /*   9  */
+		"G3 PowerBook Wallstreet",          /*  10  */
+		"1999 G3 PowerBook Lombard",         /*  11  */
+		"iBook/G3 PB Pismo/G4 PB Titanium",   /*  12  */
+		"Alluminum PowerBook",                 /*  17  */
+		N_("Wow, later than an Alluminum PB! ;-)")
+	};
+  
+	if (pmu < 8)         idx = 0;
+	else if (pmu <= 12)  idx = pmu - 8;
+	else if (pmu < 17)   idx = 0;
+	else if (pmu == 17)  idx = 5;
+	else                 idx = 6;
+	
+	snprintf(buffer, STDBUFFERLEN, "%s (PMU version: %d)", version[idx], pmu);
 }
 
 int
@@ -964,41 +997,80 @@ haveADBTrackpad ()
 	return 0;
 }
 
+int
+haveI2CAmbient ()
+{
+	struct moddata_pmac *base = &modbase_pmac;
+#if defined(DEBUG) && SIMUAMBIENT
+	return 1;
+#endif
+	
+	if ((base->machine >= 0x51) && (base->machine <= 0x57))
+		return 1;
+	return 0;
+}
+
+int
+havePMUAmbient ()
+{
+	struct moddata_pmac *base = &modbase_pmac;
+
+	if ((base->machine >= 0x58) && (base->machine <= 0x59))
+		return 1;
+	return 0;
+}
+
 /* ------------------  POWER MAC SPECIFIC KEYBOARD FUNCTIONS ------------------- */
 
 int
-keyboard_get_config (int fd)
+keyboard_get_config ()
 {
+	struct moddata_pmac *base = &modbase_pmac;
 	unsigned char ADBBuffer[ADB_BUFSIZE];
-	int n, dev, config = -1;
+	int fd, n, dev, config = -1;
 
 	if (haveADBKeyboard()) {
-		if ((dev = keyboard_get_dev (fd)) > 0) {
-			n = adb_read_reg (fd, ADBBuffer, dev, 1);
-			if (n > 0) {
-				config = 0;
-				config |= ADBBuffer[3] & 0x01 ? KEYBOARD_FNTOP : 0;
+		if ((dev = keyboard_get_dev (base->fd_adb)) > 0) {
+			if ((n = adb_read_reg (base->fd_adb, ADBBuffer, dev, 1)) > 0) {
+				config = ADBBuffer[3] & 0x01 ? KEYBOARD_FNTOP : KEYBOARD_FNBACK;
 			}
 		}
+	} else if ((fd = open (PATH_FNMODE, O_RDONLY)) >= 0) {
+		if ((n = read (fd, ADBBuffer, ADB_BUFSIZE-1)) > 0) {
+			ADBBuffer[n] = 0; /* terminate read buffer */
+			config = atoi(ADBBuffer);
+#ifdef DEBUG
+			printf ("DBG: Keyboard config read: '%s' -> %d.\n", ADBBuffer, config);			
+#endif
+		}
+		close (fd);
 	}
 	return config;
 }
 
 void
-keyboard_set_config (int fd, int config)
+keyboard_set_config (int config)
 {
+	struct moddata_pmac *base = &modbase_pmac;
 	unsigned char ADBBuffer[ADB_BUFSIZE];
-	int n, dev;
+	int fd, n, dev;
 
 	if (haveADBKeyboard()) {
-		if ((dev = keyboard_get_dev (fd)) > 0) {
-			n = adb_read_reg (fd, ADBBuffer, dev, 1);
+		if ((dev = keyboard_get_dev (base->fd_adb)) > 0) {
+			n = adb_read_reg (base->fd_adb, ADBBuffer, dev, 1);
 			if (n > 0) {
 				ADBBuffer[3] &= 0xfe;
-				ADBBuffer[3] |= (config & KEYBOARD_FNTOP) ? 0x01 : 0x00;
-				adb_write_reg (fd, ADBBuffer, n, dev, 1);
+				ADBBuffer[3] |= (config == KEYBOARD_FNTOP) ? 0x01 : 0x00;
+				adb_write_reg (base->fd_adb, ADBBuffer, n, dev, 1);
 			}
 		}
+	} else if ((fd = open (PATH_FNMODE, O_WRONLY)) >= 0) {
+		sprintf (ADBBuffer, "%d", config);
+		write (fd, ADBBuffer, strlen(ADBBuffer));
+#ifdef DEBUG
+		printf ("DBG: Keyboard config written: '%s' -> %d.\n", ADBBuffer, config);			
+#endif
+		close (fd);
 	}
 }
 
@@ -1022,15 +1094,16 @@ keyboard_get_dev (int fd)
 /* -------------------- POWER MAC SPECIFIC TRACKPAD FUNCTIONS ------------------ */
 
 int
-trackpad_get_config (int fd)
+trackpad_get_config ()
 {
+	struct moddata_pmac *base = &modbase_pmac;
 	unsigned char ADBBuffer[ADB_BUFSIZE];
 	int n, dev, config = -1;
 
 	if (haveADBTrackpad()) {
-		if ((dev = trackpad_get_dev (fd)) > 0) {
-			trackpad_set_prgmode (fd, dev, 1);
-			n = adb_read_reg (fd, ADBBuffer, dev, 2);
+		if ((dev = trackpad_get_dev (base->fd_adb)) > 0) {
+			trackpad_set_prgmode (base->fd_adb, dev, 1);
+			n = adb_read_reg (base->fd_adb, ADBBuffer, dev, 2);
 			if (n > 0) {
 				if (ADBBuffer[5] == 0xff)
 					config = TRACKPAD_LOCK;
@@ -1041,29 +1114,30 @@ trackpad_get_config (int fd)
 				else
 					config = TRACKPAD_NOTAP;
 			}
-			trackpad_set_prgmode (fd, dev, 0);
+			trackpad_set_prgmode (base->fd_adb, dev, 0);
 		}
 	}
 	return config;
 }
 
 void
-trackpad_set_config (int fd, int config)
+trackpad_set_config (int config)
 {
+	struct moddata_pmac *base = &modbase_pmac;
 	unsigned char ADBBuffer[ADB_BUFSIZE];
 	int n, dev;
 
 	if (haveADBTrackpad()) {
-		if ((dev = trackpad_get_dev (fd)) > 0) {
-			trackpad_set_prgmode (fd, dev, 1);
-			n = adb_read_reg (fd, ADBBuffer, dev, 2);
+		if ((dev = trackpad_get_dev (base->fd_adb)) > 0) {
+			trackpad_set_prgmode (base->fd_adb, dev, 1);
+			n = adb_read_reg (base->fd_adb, ADBBuffer, dev, 2);
 			if (n > 0) {
 				ADBBuffer[2] = (config < TRACKPAD_TAP) ? 0x19 : 0x99;
 				ADBBuffer[3] = (config < TRACKPAD_DRAG) ? 0x14 : 0x94;
 				ADBBuffer[5] = (config < TRACKPAD_LOCK) ? 0xb2 : 0xff;
-				adb_write_reg (fd, ADBBuffer, n, dev, 2);
+				adb_write_reg (base->fd_adb, ADBBuffer, n, dev, 2);
 			}
-			trackpad_set_prgmode (fd, dev, 0);
+			trackpad_set_prgmode (base->fd_adb, dev, 0);
 		}
 	}
 }
@@ -1106,8 +1180,8 @@ trackpad_set_prgmode (int fd, int dev, int mode)
 /* ----------------------------- LOW LEVEL ADB FUNCTIONS ---------------------------- */
 
 /* This function sends a request to the PMU via the ADB bus and reads
-    the result. If successful the count of valid data bytes in the buffer will be
-     returned, otherwise -1 */
+   the result. If successful the count of valid data bytes in the buffer will be
+   returned, otherwise -1 */
 
 int
 send_pmu_request(int fd, unsigned char *buffer, int params, ...)
@@ -1208,26 +1282,8 @@ get_int_environment(int fd)
 
 /* ----------------------------- LOW LEVEL LMU FUNCTIONS ---------------------------- */
 
-/* This function returns the a compensated ambient light sensor
- * value in percent. The reduce flickering through shadows or
- * spot lights at one of the sensors the sensor with the highest
- * value is choosen.
- */
-int
-lmu_get_ambient ()
-{
-	struct moddata_pmac *base = &modbase_pmac;
-	int rc;
-
-	rc = (int) (((lmu_get_raw_ambient(base->i2c_bus_lmu) - base->lcdfeedback) 
-				* 100) / AMBIENTMAX);
-	if (rc < 0)	  rc = 0;
-	if (rc > 100) rc = 100;
-	return rc;
-}
-
 /* This function tries to find the I2C device that controls the keyboard
- * illumination and the ambient light sensor used in alluminum PowerBooks
+ * illumination and the ambient light sensor used in some alluminum PowerBooks
  * It returns the device number on success, otherwise a negative error code.
  * The loop starts with device 4 because that's the correct device on
  * 17" alluminum PowerBooks. Device 0 could be errornous detected because
@@ -1235,7 +1291,7 @@ lmu_get_ambient ()
  * device.
  */
 int
-lmu_find_bus ()
+findI2CBus ()
 {
 #if defined(DEBUG) && SIMUAMBIENT
 	int rc = 99; /* dummy file handle for i2c bus, will hopefully never be used */
@@ -1266,68 +1322,61 @@ lmu_find_bus ()
 	return rc;
 }
 
-void
-lmu_set_illumination (int dev, unsigned short level)
-{
-#if !(defined(DEBUG) && SIMUAMBIENT)
-	char i2cdevice[20];
-	int fd;
-	__u8 buf[3];
-
-	if (dev >= 0) {
-		if (level > 15) level = 15; 	/* check bounds */
-		buf[0] = 0x01;   /* i2c register */
-		buf[1] = (__u8) (( ( level * 255 ) & 0xFF00 ) >> 8 );
-		buf[2] = (__u8) (( ( level * 255 ) & 0x00FF ));
-
-		snprintf(i2cdevice, 19, "/dev/i2c-%d", dev );
-		if ((fd = open (i2cdevice, O_RDWR)) >= 0 ) {
-			if (ioctl (fd, I2C_SLAVE, LMU_ADDR) >= 0 )
-				write (fd, buf, 3);
-			close (fd);
-		}
-	}
-#else
-	printf("KBD Level: %2d at %2d%% Ambient\n",
-		level, lmu_get_ambient());
-	fflush(stdout);
-#endif
-}
-
-/* This function reads the ambient light sensor. New aluminum
- * PowerBooks have two of them left and right below the speaker
- * covers.
- * This function asks both sensors and the highest value is
- * returned
+/* This function returns the a compensated ambient light sensor
+ * value in percent. The reduce flickering through shadows or
+ * spot lights at one of the sensors the sensor with the highest
+ * value is choosen.
  */
 int
-lmu_get_raw_ambient (int dev)
+getAmbient ()
 {
-#if defined(DEBUG) && SIMUAMBIENT
 	struct moddata_pmac *base = &modbase_pmac;
-	int ambient, level;
+	int rc, ambientmax;
+
+	ambientmax = haveI2CAmbient() ? 1600 : 2048;
+	rc = (int) (((getRawAmbient() - base->lcdfeedback) 
+				* 100) / ambientmax);
+
+	if (rc < 0)	  rc = 0;
+	if (rc > 100) rc = 100;
+	return rc;
+}
+
+/* The first ambient light sensor was connected to the I2C
+ * Bus but recent PowerBooks 5.8 and 5.9 have a new ambient
+ * light sensor connected to the PMU. This functions reads
+ * the current sensor value from the right interface.
+ */
+int
+getRawAmbient ()
+{
+	struct moddata_pmac *base = &modbase_pmac;
+	unsigned char buf[ADB_BUFSIZE];
+	int fd, rc = -1, rc2;
+	
+#if defined(DEBUG) && SIMUAMBIENT
+	int ambient, level, idx;
 
 	/* this array contains sensor values depending on the
 	 * LCD brightness without additional ambient light.
 	 * They simulate the LCD brightness influence on the
 	 * light sensors and are for debugging only.
+	 *
+	 * This function simulates an I2C ambient sensor and
+	 * will also work with more than 16 LCD brightness
+	 * levels.
 	 */
 	int asens[] = {2, 182, 217, 248, 309, 337, 381, 423, 461,
 		           499, 538, 576, 620, 1072, 1073, 1077};
 		       
 	level = pmac_get_lcdbacklight();
-	ambient = asens[level == -1 ? 0 : level] + base->ambient;
-	if (ambient > AMBIENTMAX)
-		ambient = AMBIENTMAX;
-	
+	idx = level * sizeof(asens) / sizeof(int) / (BACKLIGHTMAX + 1);
+	ambient = asens[level == -1 ? 0 : idx] + base->ambient;
+	if (ambient > 1600) ambient = 1600;
 	return ambient;
-#else
-	char i2cdevice[20], buf[4];
-	int fd, rc = -1, rc2;
-
-	if (dev >= 0) {
-		snprintf(i2cdevice, 19, "/dev/i2c-%d", dev );
-		if ((fd = open (i2cdevice, O_RDWR)) >= 0 ) {
+#else	
+	if ((haveI2CAmbient()) && (base->i2cdev[0])) {
+		if ((fd = open (base->i2cdev, O_RDWR)) >= 0 ) {
 			if (ioctl (fd, I2C_SLAVE, LMU_ADDR) >= 0 )
 				if (read (fd, buf, 4 ) == 4) {
 					rc  = (buf[0] << 8) | buf[1];
@@ -1337,10 +1386,56 @@ lmu_get_raw_ambient (int dev)
 			close (fd);
 		}
 	}
+	
+	if (havePMUAmbient()) {
+		if (send_pmu_request (base->fd_adb, buf, 2, 0x4F, 1) == 5) {
+			rc  = (buf[2] << 8) | buf[1];
+			rc2 = (buf[4] << 8) | buf[3];
+			rc  = rc > rc2 ? rc : rc2; 
+		}
+	}
 	return rc;
 #endif
 }
 
+void
+setKBDIllumination (unsigned short level)
+{
+#if !(defined(DEBUG) && SIMUAMBIENT)
+	struct moddata_pmac *base = &modbase_pmac;
+	unsigned char buf[ADB_BUFSIZE];
+	int fd;
+
+	if (level > KEYBLIGHTMAX) level = KEYBLIGHTMAX; /* check bounds */
+	
+	if ((haveI2CAmbient ()) && base->i2cdev[0]) {
+		buf[0] = 0x01;   /* i2c register */
+		buf[1] = (unsigned char) (( ( level * 255 ) & 0xFF00 ) >> 8 );
+		buf[2] = (unsigned char) (( ( level * 255 ) & 0x00FF ));
+
+		if ((fd = open (base->i2cdev, O_RDWR)) >= 0 ) {
+			if (ioctl (fd, I2C_SLAVE, LMU_ADDR) >= 0 )
+				write (fd, buf, 3);
+			close (fd);
+		}
+	}
+
+	if (havePMUAmbient ()) {
+		/* The keyboard illumination connected to the PMU has 256 brightness
+		 * steps and because pbbuttonsd handles only 16 steps in general, we
+		 * use only the high nibble here.
+		 */
+		level = level * 255 / KEYBLIGHTMAX;
+		send_pmu_request (base->fd_adb, buf, 4, 0x4F, 0, 0, level);
+	}
+#else
+	printf("KBD Level: %2d at %2d%% Ambient\n",
+		level, getAmbient());
+	fflush(stdout);
+#endif
+}
+
+
 /* --------------------- BATTERY LOGFILE DEBUG FUNCTIONS --------------------- */
 int
 batlog_setup()
