@@ -41,6 +41,9 @@
 #include "support.h"
 #include "debug.h"
 
+#ifdef WITH_IBAM
+#  include "ibam_stub.h"
+#endif
 
 /* --- private module data structure of module powermanagement --- */
 struct moddata_power {
@@ -50,7 +53,8 @@ struct moddata_power {
 	struct powerprofile *activeProfile;
 	int policy;        /* current power policy */
 	int powersource;    /* REDUNDANT copy from module pmac */
-	int timeremaining;   /* REDUNDANT copy from module pmac */
+	int timeremaining;   /* REDUNDANT copy from module pmac
+							or filled by IBaM */
 	int mode;             /* module mode: awake, sleep, dim. */
 	int sleeptime;         /* counter */
 	int sleepcountdown;     /* another counter */
@@ -69,6 +73,10 @@ struct moddata_power {
 	int netload_period;      /* period of time for that net activity mut be below netload */
 	char *netload_dev;        /* device to observe for netload sleeplock */
 	char *script_pmcs;         /* command to call at certain event */
+#ifdef WITH_IBAM
+	char *ibam_datadir;
+	int chargetime;     /* time until battery is fully charged */
+#endif
 } modbase_power;
 
 /* --- interface functions of module powermanagement --- */
@@ -83,13 +91,21 @@ power_init ()
 	struct moddata_power *base = &modbase_power;
 	static char scriptbuffer1[STDBUFFERLEN];
 	static char scriptbuffer2[STDBUFFERLEN];
-	int sid;
+#ifdef WITH_IBAM
+	static char scriptbuffer3[STDBUFFERLEN];
+#endif
+	int sid, err;
 
 	bzero (base, sizeof (struct moddata_power));
 	bzero (scriptbuffer1, STDBUFFERLEN);  /* no default for script_pmcs */
-	sprintf (scriptbuffer2, DEFAULT_NETLOAD_DEV);
 	base->script_pmcs  = scriptbuffer1;
+	sprintf (scriptbuffer2, DEFAULT_NETLOAD_DEV);
 	base->netload_dev  = scriptbuffer2;
+#ifdef WITH_IBAM
+	base->ibam_datadir  = scriptbuffer3;
+	if ((err = copy_path (DEFAULT_IBAM_DATADIR, base->ibam_datadir, TYPE_DIR, CPFLG_PBBONLY))) 
+		return err;
+#endif
 	base->mode         = MODE_AWAKE;
 	base->policy       = POLICY_PERFORMANCE;
 	
@@ -162,9 +178,14 @@ power_init ()
 				N_("time in seconds"));
 		registerCfgOptionString (sid, "NETLoad_device", TAG_NETLOADDEV, 0,
 				NULL);
+#ifdef WITH_IBAM
+		registerCfgOptionString (sid, "IBAM_DataDir", TAG_IBAMDATADIR, 0,
+				NULL);
+#endif
 	}
 
 	ipc_protect_tag (TAG_SCRIPTPMCS);  /* tags only for privileged use */
+	ipc_protect_tag (TAG_IBAMDATADIR);
 
 	register_function (QUERYQUEUE, power_query);
 	register_function (CONFIGQUEUE, power_configure);
@@ -229,6 +250,15 @@ power_open (struct tagitem *taglist)
 				  will send a TAG_POWERCHANGE to correct it and start the script
 				  Not elegant but works */
 	
+#ifdef WITH_IBAM
+	base->timeremaining = -1; /* will be updated from timer1000 */
+	base->chargetime    = -1;
+	if (!ibam_init(base->ibam_datadir)) {  /* create ibam object */
+		print_msg (PBB_ERR, _("Can't create IBaM object, out of memory.\n"));
+		return E_NOMEM;
+	}
+#endif
+		
 	register_function (KBDQUEUE, power_keyboard);
 	register_function (MOUSEQUEUE, power_mouse);
 	register_function (T100QUEUE, power_timer);
@@ -239,6 +269,9 @@ power_open (struct tagitem *taglist)
 int
 power_close ()
 {
+#ifdef WITH_IBAM
+	ibam_exit();
+#endif
 	return 0;
 }
 
@@ -306,11 +339,15 @@ power_sync ()
 	taglist_init (args);
 	taglist_add (args, TAG_SLEEPSUPPORTED, 0);
 	taglist_add (args, TAG_POWERSOURCE, 0);
+#ifndef WITH_IBAM
 	taglist_add (args, TAG_TIMEREMAINING, -1);
+#endif
 	process_queue (QUERYQUEUE, args);
 	base->flags.sleep_supported = tagfind (args, TAG_SLEEPSUPPORTED, 0);
 	base->powersource = tagfind (args, TAG_POWERSOURCE, 0);
+#ifndef WITH_IBAM
 	base->timeremaining = tagfind (args, TAG_TIMEREMAINING, -1);
+#endif
 	base->activeProfile = base->powersource ? &base->onAC : &base->onBattery;
 }
 
@@ -351,6 +388,15 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			} else
 				taglist->data = (long) base->script_pmcs;
 			break;
+#ifdef WITH_IBAM
+		case TAG_IBAMDATADIR:
+			if (cfgure) {
+				if ((err = copy_path ((char *) taglist->data, base->ibam_datadir, TYPE_DIR, CPFLG_PBBONLY)))
+					tagerror (taglist, err);
+			} else
+				taglist->data = (long) base->ibam_datadir;
+			break;
+#endif
 			
 		/* Tags for power profiles */	
 		case TAG_ONAC_POLICY:
@@ -560,12 +606,23 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 					power_suspend (base->activeProfile->coveraction);
 			}
 			break;
+#ifndef WITH_IBAM
 		case TAG_TIMECHANGED:  /* private tag */
 			if (cfgure) {
 				base->timeremaining = taglist->data;
 				power_check_battery (base->timeremaining, base->powersource);
 			}
 			break;
+#else
+		case TAG_TIMEREMAINING:
+			if (cfgure)	tagerror (taglist, E_NOWRITE);
+			else		taglist->data = base->timeremaining;
+			break;
+		case TAG_CHARGETIME:
+			if (cfgure)	tagerror (taglist, E_NOWRITE);
+			else		taglist->data = base->chargetime;
+			break;
+#endif
 		case TAG_PREPAREFORSLEEP:  /* private tag */
 			/* PMCS-script will be called in power_suspend() so that it's nothing left to do here */
 			break;
@@ -668,7 +725,7 @@ power_timer1000 (struct tagitem *taglist)
 {
 	struct moddata_power *base = &modbase_power;
 	static int cputimer = 0, nettimer = 0, beeptimer = 0;
-	int cpuload = 0, netload = 0;
+	int cpuload = 0, netload = 0, time;
 
 	if (base->flags.cpuload_enable) {
 		cpuload = power_read_cpu ();
@@ -712,6 +769,23 @@ power_timer1000 (struct tagitem *taglist)
 	} else
 		beeptimer = 0;
 
+#ifdef WITH_IBAM
+	ibam_recordProfile();  /* keep battery statistics up-to-date */
+	time = ibam_getBatteryTimeAdaptive() * 60; /* IBAM delivers minutes, we need seconds */
+	if (time != base->timeremaining) {
+		base->timeremaining = time;
+		power_check_battery (base->timeremaining, base->powersource);
+	}
+	base->chargetime = ibam_getChargeTimeAdaptive() * 60;
+#  if defined(DEBUG) && IBAM
+	int ctime = base->chargetime;
+	printf("DBG: %s: Charge adaptive time: %2d:%2d, discharging adaptive time: %2d:%02d\r",
+			base->powersource ? "AC " : "BAT",
+			ctime/3600, (ctime%3600)/60, time/3600, (time%3600)/60);
+	fflush(stdout);
+#  endif
+#endif
+	
 #if defined(DEBUG) && SLEEPLOCKS
 	printf ("cpu: ");
 	if (base->flags.cpuload_enable)
@@ -846,6 +920,7 @@ power_suspend (int action)
 		return;
 	
 	base->mode = MODE_SUSPEND;
+	sync();
 
 	switch (action) {
 	case ACTION_TODISK:
