@@ -152,6 +152,9 @@ system_handle_tags (int cfgure, struct tagitem *taglist)
 #endif
 #ifdef WITH_ALSA
 				val |= SYSINFO_HAVEALSA;
+#endif
+#ifdef WITH_IBAM
+				val |= SYSINFO_HAVEIBAM;
 #endif
 				taglist->data = val;
 			}
