@@ -23,22 +23,18 @@
 #include <signal.h>
 #include <sys/types.h>
 
+#include <glib.h>
 #include <pbb.h>
 
 #include "init.h"
 #include "input_manager.h"
 #include "gettext_macros.h"
 
-volatile sig_atomic_t prgexit = 0;
-volatile sig_atomic_t reloadcfg = 0;
-volatile sig_atomic_t reset = 0;
-
 int main(int argc, char *argv[])
 {
 	struct serverdata sdata, *sd = &sdata;
-	int val, maxfd, rc = 1;
-	fd_set inset;
-	struct timeval tv;
+	gint timerid10, timerid100, timerid1000;
+	int rc = 1;
 	long tval;
 
 #ifdef ENABLE_NLS
@@ -50,39 +46,31 @@ int main(int argc, char *argv[])
 	init_serverdata (sd);        /* fill serverdata structure with default values */
 	if ((rc = evaluate_args(sd, argc, argv)) == 0) {  /* evaluate command line */
 		if ((rc = init_modules (sd)) == 0) {      /* call known modules for initialisation */
-			if (!install_sighandler ()) {
-				tval = process_queue_single (QUERYQUEUE, TAG_IDENTITY, 0);
-				if (!sd->quiet && tval)
-					printf ("%s %s: %s\n", PACKAGE, VERSION, (char*) tval);
+			tval = process_queue_single (QUERYQUEUE, TAG_IDENTITY, 0);
+			if (!sd->quiet && tval)
+				printf ("%s %s: %s\n", PACKAGE, VERSION, (char*) tval);
 
-				if ((create_daemon ()) == 0) {
-					if (tval) syslog (LOG_INFO, "INFO: %s %s: %s\n", PACKAGE, VERSION, (char*) tval);
-					tval = process_queue_single (QUERYQUEUE, TAG_POWERSOURCE, 0);
-					process_queue_single (CONFIGQUEUE, TAG_POWERCHANGED, tval); /* start profile changed script */
+			if ((create_daemon ()) == 0) {
+				if (tval) syslog (LOG_INFO, "INFO: %s %s: %s\n", PACKAGE, VERSION, (char*) tval);
+				tval = process_queue_single (QUERYQUEUE, TAG_POWERSOURCE, 0);
+				process_queue_single (CONFIGQUEUE, TAG_POWERCHANGED, tval); /* start profile changed script */
 
-					while (prgexit == 0) {
-						tv.tv_sec = 0; tv.tv_usec = 10000;  /* 10ms polling timeout */
-						maxfd = create_fdset(&inset);    /* prepare filehandles for select */
-						if ((val = select (maxfd+1, &inset, NULL, NULL, &tv)) >= 0) {
-							if (val == 0)
-								call_timertick();        /* call timer handler */
-							else
-								call_inputhandler(&inset); /* call input handler */
-						}
-						if (reloadcfg == 1) {      /* SIGHUP received ? */
-							scanEventDevices ();   /* look which HID are available */
-							reconfig_modules (sd); /* load config file again */
-							reloadcfg = 0;
-						}
-						if (reset == 1) {          /* SIGUSR1 received ? */
-							process_queue_single (CONFIGQUEUE, TAG_REINIT, 0);
-							reset = 0;
-						}
-					}
-				}
-				rc = 0;
+				sd->mainloop = g_main_loop_new (NULL, TRUE);
+				installSigHandler (sd);
+
+				timerid10 = g_timeout_add (10, cb_timer10, NULL);
+				timerid100 = g_timeout_add (100, cb_timer100, NULL);
+				timerid1000 = g_timeout_add (1000, cb_timer1000, NULL);
+					
+				g_main_loop_run (sd->mainloop);
+				
+				g_source_remove (timerid10);
+				g_source_remove (timerid100);
+				g_source_remove (timerid1000);
+				g_main_loop_unref (sd->mainloop);
 			}
 			exit_modules (sd);  /* let modules cleanup their stuff */
+			rc = 0;
 		}
 	}
 	exit_libpbb();            /* cleanup library */
